@@ -7,7 +7,11 @@ Classes used to help make parsing more modular. These include the NexusBase
 class and its children.
 """
 
+# We've gotta access the _value attribute on some NXobjects.
+# pylint: disable=protected-access
 
+
+import json
 import os
 from typing import List
 from abc import abstractmethod
@@ -146,7 +150,8 @@ class I07Nexus(NexusBase):
     This class extends NexusBase with methods useful for scraping information
     from nexus files produced at the I07 beamline at Diamond.
     """
-    excalibur_detector = "Excalibur"
+    excalibur_detector_2021 = "excroi"
+    excalibur_04_2022 = "exr"
 
     @property
     def local_data_path(self) -> str:
@@ -165,10 +170,13 @@ class I07Nexus(NexusBase):
     @property
     def detector_name(self) -> str:
         """
-        Returns the name of the detector that we're using.
+        Returns the name of the detector that we're using. Because life sucks,
+        this is a function of time.
         """
         if "excroi" in self.entry:
-            return I07Nexus.excalibur_detector
+            return I07Nexus.excalibur_detector_2021
+        if "exr" in self.entry:
+            return I07Nexus.excalibur_04_2022
         # Couldn't recognise the detector.
         raise NotImplementedError()
 
@@ -215,7 +223,30 @@ class I07Nexus(NexusBase):
         Currently there is nothing better to do than assume that this is a list
         of length 1.
         """
-        return [self._get_ith_region(i=1)]
+        if self.detector_name == I07Nexus.excalibur_detector_2021:
+            return [self._get_ith_region(i=1)]
+        if self.detector_name == I07Nexus.excalibur_04_2022:
+            # Make sure our code executes for bytes and strings.
+            try:
+                json_str = self.instrument[
+                    "ex_rois/excalibur_ROIs"]._value.decode("utf-8")
+            except AttributeError:
+                json_str = self.instrument[
+                    "ex_rois/excalibur_ROIs"]._value
+            # This is badly formatted and cant be loaded by the json lib. We
+            # need to make a series of modifications.
+            json_str = json_str.replace('u', '')
+            json_str = json_str.replace("'", '"')
+            json_str = json_str.replace('x', '"x"')
+            json_str = json_str.replace('y', '"y"')
+            json_str = json_str.replace('width', '"width"')
+            json_str = json_str.replace('height', '"height"')
+            json_str = json_str.replace('angle', '"angle"')
+
+            roi_dict = json.loads(json_str)
+            return [Region.from_dict(roi_dict['Region_1'])]
+
+        raise NotImplementedError()
 
     @property
     def background_regions(self) -> List[Region]:
@@ -224,8 +255,32 @@ class I07Nexus(NexusBase):
         Currently we just ignore the zeroth region and call the rest of them
         background regions.
         """
-        return [self._get_ith_region(i)
-                for i in range(2, self._number_of_regions+1)]
+        if self.detector_name == I07Nexus.excalibur_detector_2021:
+            return [self._get_ith_region(i)
+                    for i in range(2, self._number_of_regions+1)]
+        if self.detector_name == I07Nexus.excalibur_04_2022:
+            # Make sure our code executes for bytes and strings.
+            try:
+                json_str = self.instrument[
+                    "ex_rois/excalibur_ROIs"]._value.decode("utf-8")
+            except AttributeError:
+                json_str = self.instrument[
+                    "ex_rois/excalibur_ROIs"]._value
+            # This is badly formatted and cant be loaded by the json lib. We
+            # need to make a series of modifications.
+            json_str = json_str.replace('u', '')
+            json_str = json_str.replace("'", '"')
+            json_str = json_str.replace('x', '"x"')
+            json_str = json_str.replace('y', '"y"')
+            json_str = json_str.replace('width', '"width"')
+            json_str = json_str.replace('height', '"height"')
+            json_str = json_str.replace('angle', '"angle"')
+
+            roi_dict = json.loads(json_str)
+            bkg_roi_list = list(roi_dict.values())[1:]
+            return [Region.from_dict(x) for x in bkg_roi_list]
+
+        raise NotImplementedError()
 
     @property
     def probe_energy(self):
@@ -463,7 +518,9 @@ def i07_nxs_parser(file_path: str):
 
     # Load the images, taking a transpose if necessary (because which axis is
     # x and which is why is determined by fast vs slow detector axes in memory).
-    if i07_nxs.detector_name == I07Nexus.excalibur_detector:
+    if i07_nxs.detector_name in [
+            I07Nexus.excalibur_detector_2021,
+            I07Nexus.excalibur_04_2022]:
         images = load_images_from_h5(i07_nxs.local_data_path, transpose=True)
 
     # The dependent variable.
@@ -483,6 +540,9 @@ def i07_nxs_parser(file_path: str):
     elif i07_nxs.default_axis_type == 'tth':
         data = Data(rough_intensity, rough_intensity_e, i07_nxs.probe_energy,
                     theta=axis/2)
+    else:
+        raise NotImplementedError(
+            f"{i07_nxs.default_axis_type} is not a supported axis type.")
 
     # Returns the Scan2D object
     return Scan2D(data, i07_nxs, images)
